"use client";
import * as z from "zod";
import ApiRequest from "@/services/apiRequest";
import { toast } from "@/components/ui/use-toast";
import * as React from "react";
import { useForm } from "react-hook-form";
import { zodResolver } from "@hookform/resolvers/zod";
import { Form, FormControl, FormField, FormItem, FormLabel, FormMessage } from "@/components/ui/form";
import { Select, SelectContent, SelectItem, SelectTrigger, SelectValue } from "@/components/ui/select";
import { Button } from "@/components/ui/button";
import { useEffect, useState } from "react";
import type { IUserWalletCryptos } from "@/interfaces/crypto.interface";
import type { IApiUserAssetsRes } from "@/interfaces/api.interface";
import { Slider } from "@/components/ui/slider";
import { DollarSign, Equal, X } from "lucide-react";
import { Card } from "@/components/ui/card";

export function SellForm() {
	const [currentWallet, setCurrentWallet] = useState<IUserWalletCryptos[]>();
	const [sliderValue, setSliderValue] = useState(0);
	const [quantity, setQuantity] = useState(0);
	const [selectedCrypto, setSelectedCrypto] = useState<string>();

	useEffect(() => {
		async function fetchAssets() {
			const res = await ApiRequest.authenticated.get.json<IApiUserAssetsRes>("user/my-assets");
			setCurrentWallet(res.data.UserHasCrypto);
		}
		fetchAssets();
	}, []);

	function getMaxSharesForCrypto(cryptoId: string) {
		return currentWallet?.find((crypto) => crypto.Crypto.id === cryptoId)?.amount || 0;
	}

	function getCryptoValue(cryptoId: string) {
		return currentWallet?.find((crypto) => crypto.Crypto.id === cryptoId)?.Crypto.value || 0;
	}

	function getCryptoName(cryptoId: string) {
		return currentWallet?.find((crypto) => crypto.Crypto.id === cryptoId)?.Crypto.name || "";
	}

	const sellToUserSchema = z.object({
		cryptoId: z.string({ required_error: "You should select a crypto from your wallet." }).uuid(),
		amount: z.number({
			required_error: "You should select an amount of the assets that you want to sell.",
		}).max(sliderValue),
	});

	async function onSellToUserSubmit(data: z.infer<typeof sellToUserSchema>) {
		const res = await ApiRequest.authenticated.post.json("offer/create", {
			id_crypto: data.cryptoId,
			amount: data.amount,
		});

		if (res.status !== 201) {
			toast({
				title: "An error occurred!",
				description: (
					<pre className="mt-2 w-[340px] rounded-md bg-slate-950 p-4">
            <code className="text-white text-wrap">{JSON.stringify(res.statusText, null, 2)}</code>
          </pre>
				),
			});
			return;
		}

		console.log(res);
		toast({
			title: "Transaction accepted.",
			description: <p>The page is going to reload.</p>,
		});
		setTimeout(() => location.reload(), 1_500);

		toast({
			title: "You submitted the following values:",
			description: (
				<pre className="mt-2 w-[340px] rounded-md bg-slate-950 p-4">
          <code className="text-white">{JSON.stringify(data, null, 2)}</code>
        </pre>
			),
		});
	}

	const sellToUserForm = useForm<z.infer<typeof sellToUserSchema>>({
		resolver: zodResolver(sellToUserSchema),
	});

	if (!currentWallet) {
		return (
			<Card>
				<div className="flex items-center justify-center p-2">
					<p>Loading...</p>
				</div>
			</Card>
		);
	}

	return (
		<Form {...sellToUserForm}>
			<form onSubmit={sellToUserForm.handleSubmit(onSellToUserSubmit)} className="w-full space-y-6">
				<FormField
					control={sellToUserForm.control}
					name="cryptoId"
					render={({ field }) => (
						<FormItem
							onChange={(event) => {
								// @ts-ignore
								const maxShares = getMaxSharesForCrypto(event.target.value as string);
								setSliderValue(maxShares);
								setQuantity(maxShares);
								// @ts-ignore
								setSelectedCrypto(event.target.value as string);
							}}
						>
							<FormLabel>Sell shares of your wallet</FormLabel>
							<Select
								onValueChange={(val) => {
									const maxShares = getMaxSharesForCrypto(val);
									setSliderValue(maxShares);
									setQuantity(1);
									setSelectedCrypto(val);
									field.onChange(val);
								}}
							>
								<FormControl>
									<SelectTrigger>
										<SelectValue placeholder="Select a crypto from your wallet." />
									</SelectTrigger>
								</FormControl>
								<SelectContent>
									{currentWallet?.map((crypto) => (
										<SelectItem value={crypto.Crypto.id} key={crypto.Crypto.id}>
											{`${crypto.amount}x ${crypto.Crypto.name} - ${crypto.Crypto.value}$/u`}
										</SelectItem>
									))}
								</SelectContent>
							</Select>
							<FormMessage />
						</FormItem>
					)}
				/>
				<FormField
					control={sellToUserForm.control}
					name="amount"
					render={({ field }) => (
						<FormItem hidden={!selectedCrypto}>
							<FormLabel>Quantity</FormLabel>
							<div className="flex items-center justify-between gap-2 p-2">
								<p className="bg-accent text-accent-foreground p-1 rounded">{field.value}</p>
								<Slider
									form="amount"
									onValueChange={(val) => {
										const singleValue = val[0];
										setQuantity(singleValue as number);
										field.onChange(singleValue);
									}}
									value={[quantity]}
									min={1}
									max={sliderValue}
									step={1}
								/>
							</div>
							<FormMessage />
						</FormItem>
					)}
				/>
				{selectedCrypto && (
					<div className="flex items-center justify-center p-2 gap-2">
						<p>{getCryptoName(selectedCrypto)}</p>
						<X />
						<p>{quantity}</p>
						<Equal />
						<em className="bg-secondary/35 p-1 rounded">
							{(quantity * getCryptoValue(selectedCrypto)).toLocaleString("en-US")}
						</em>
						<DollarSign />
					</div>
				)}
				<Button type="submit" disabled={!quantity}>
					Place sell order
				</Button>
			</form>
		</Form>
	);
}